@@ -1,5 +1,7 @@
 ## library("graph")
 set.seed(0x12a9b)
+   library(graph)
+   library(RUnit)
 
 randBAMGraph <- function(numNodes = 10 , numEdges = 10)
 {
@@ -190,14 +192,29 @@ test_BAM_edgeMatrix <- function() {
       checkEquals(em[2,], c(1, 2, 3, 3, 4, 5))
 }
 
+test_BAM_adjacencyMatrix <- function() {
+      g1 <- make_smallBAM()
+      checkEquals(edgemode(g1), "directed")
+      checkEquals(nodes(g1),  c("a","b","c","x","y"))
+      am <- adjacencyMatrix(g1)
+      checkEquals(rownames(am), nodes(g1))
+      checkEquals(colnames(am), nodes(g1))
+      checkEquals(as.integer(am["a",]), c(0, 1, 1, 1, 0))
+      checkEquals(as.integer(am["b",]), c(0, 0, 0, 0, 0))
+      checkEquals(as.integer(am["c",]), c(1, 0, 0, 0, 0))
+      checkEquals(as.integer(am["x",]), c(0, 0, 1, 0, 1))
+      checkEquals(as.integer(am["y",]), c(0, 0, 0, 0, 0))
+
+}
+
 test_BAM_removeEdge_unknown_nodes <- function()
 {
     g1 <- make_smallBAM()
-    checkException(removeEdge("a", "q", g1))
-    checkException(removeEdge("q", "a", g1))
-    checkException(removeEdge("a", c("q", "aa", "tt"), g1))
+    checkException(removeEdge("a", "q", g1), silent=TRUE)
+    checkException(removeEdge("q", "a", g1), silent=TRUE)
+    checkException(removeEdge("a", c("q", "aa", "tt"), g1), silent=TRUE)
     checkException(removeEdge(c("a", "q", "tt", "aa"),
-                              c("a", "q", "aa", "tt"), g1))
+                              c("a", "q", "aa", "tt"), g1), silent=TRUE)
 }
 
 test_BAM_removeEdge <- function()
@@ -207,7 +224,7 @@ test_BAM_removeEdge <- function()
     c0 <- character(0)
     checkEquals(edges(g1), edges(removeEdge(c0, c0, g1)))
     ## there is no y => a edge, throw error
-    checkException(removeEdge("y", "a", g1))
+    checkException(removeEdge("y", "a", g1), silent=TRUE)
 
     g2 <- removeEdge("c", "a", g1)
     checkEquals(list(c=character(0)), edges(g2, "c"))
@@ -244,7 +261,7 @@ test_BAMSmall_edgeData <- function(){
       checkEquals(names(vals), tmp)
       checkEquals( as.numeric(vals), c(3.4, 2.6, 1.7))
 
-      checkException(eg <- edgeData(g1, "a", attr="weightsss"))
+      checkException(eg <- edgeData(g1, "a", attr="weightsss"), silent=TRUE)
 
       eg <- edgeData(g1, "a", "b", attr="weight")
       tmp <- paste("a", "b", sep = "|")
@@ -597,7 +614,7 @@ test_BAM_Intersect_EmptyNodes <- function() {
 test_BAM_isAdjacent <- function()
 {
      from = c("a", "d", "d", "b", "a")
-     to = c("b", "a", "d", "c", "c")
+     to   = c("b", "a", "d", "c", "c")
      weight= c(1.5, 2.1, 3.4, 4.1, 5.6)
      df <- data.frame(from, to, weight)
      gd <- graphBAM(df, nodes="e", edgemode = "directed")
@@ -685,7 +702,7 @@ test_BAM_Union_Mixed <- function() {
     g2 <- graphBAM(df, nodes = c("a","b","c", "d", "x", "y", "z"), 
             edgemode = "directed")
 
-    checkException(g <- graphUnion(g1,g2))
+    checkException(g <- graphUnion(g1,g2), silent=TRUE)
 }
 
 test_BAM_inEdges <- function()
@@ -718,7 +735,7 @@ test_BAM_directed_attrs <- function() {
     df <- data.frame(from, to, weight)
     bam <- graphBAM(df, edgemode = "directed")
     
-    checkException(edgeData(bam,from="a", attr="code"))
+    checkException(edgeData(bam,from="a", attr="code"), silent=TRUE)
     edgeDataDefaults(bam, attr ="weight") <- 1
     edgeDataDefaults(bam, attr = "code") <- "plain"
 
@@ -746,7 +763,7 @@ test_BAM_undirected_attrs <- function() {
     weight = c(2, 1, 3, 4, 5)
     df <- data.frame(from, to, weight)
     bam <- graphBAM(df, edgemode = "undirected")
-    checkException(edgeData(bam,from="a", attr="code"))
+    checkException(edgeData(bam,from="a", attr="code"), silent=TRUE)
 
     edgeDataDefaults(bam, attr = "weight") <- 1
     edgeDataDefaults(bam, attr = "code") <- "plain"
@@ -807,7 +824,7 @@ test_graphBAM_detailed_Attribute_Intersection <- function() {
     target <- structure( c("red", "blue", NA), names = nms)
     checkEquals(target, unlist(attColor))
 
-    checkException(edgeData(g, attr = "type"))
+    checkException(edgeData(g, attr = "type"), silent=TRUE)
 
     weightFun <- function(x, y) {
         return(x +y )
@@ -1152,7 +1169,7 @@ test_graphBAM_S4_Attribute_Intersection <- function() {
     target <- structure( c(myColor("red"), myColor("blue"), NA), names = nms)
     checkEquals(target, unlist(attColor))
 
-    checkException(edgeData(g, attr = "type"))
+    checkException(edgeData(g, attr = "type"), silent=TRUE)
 
     weightFun <- function(x, y) {
         return(x + y )
@@ -1174,7 +1191,7 @@ test_graphBAM_S4_Attribute_Intersection <- function() {
     nms <- paste(c("a", "b", "d"),  c("b", "c", "x"), sep = "|")
     target <- structure( c("white", "black", "black"), names = nms)
     checkEquals(target, unlist(attColor))
-    checkException(edgeData(g, attr = "type"))
+    checkException(edgeData(g, attr = "type"), silent=TRUE)
 
 }
 
@@ -1276,15 +1293,42 @@ test_graphBAM_addNode1 <- function(){
 
     gr <- addNode(c("q", "ss"), g)
     current <- nodeData(gr, attr = "color")
-    target <- as.list(structure( c("green", "pink", "pink", "red", "pink", "pink", "pink", "pink"), 
-                            names = c("a", "b", "c", "d", "q", "ss", "x", "y")))
-    checkEquals(target, current)
-    
+
+    target <- c("green", "pink", "pink", "red", "pink", "pink", "pink", "pink")
+    names(target) <- c("a", "b", "c", "d", "q", "ss", "x", "y")
+    checkTrue(all(current[sort(names(current))] == target[sort(names(target))]))
+
     current <- nodeData(gr, attr = "type")
-    target <- as.list(structure( c("high", "med", "unknown", "low", "unknown",
-                "unknown", "unknown", "high"), 
-                            names = c("a", "b", "c", "d", "q", "ss", "x", "y")))
-    checkEquals(target, current)
+    target <- c("high", "med", "unknown", "low", "unknown", "unknown", "unknown", "high")
+    names(target) <- c("a", "b", "c", "d", "q", "ss", "x", "y")
+    checkTrue(all(current[sort(names(current))] == target[sort(names(target))]))
+
+}
+
+test_graphBAM_addNode1 <- function(){
+
+    from = c("a", "b", "d", "d")
+    to   = c("b", "c", "y", "x")
+    weight=c(2.2, 2.0, 0.4, 0.2)
+    df <- data.frame(from, to, weight)
+    g <- graphBAM(df, edgemode = "directed")
+    nodeDataDefaults(g, attr="color") <- "pink"
+    nodeData(g, n = c("d","a"), attr = "color") <- c("red", "green")
+    nodeDataDefaults(g, attr="type") <- "unknown"
+    nodeData(g, n = c("a", "b", "y", "d"), attr = "type") <- c("high", "med", "high", "low") 
+
+    gr <- addNode(c("q", "ss"), g)
+    current <- nodeData(gr, attr = "color")
+
+    target <- c("green", "pink", "pink", "red", "pink", "pink", "pink", "pink")
+    names(target) <- c("a", "b", "c", "d", "q", "ss", "x", "y")
+    checkTrue(all(current[sort(names(current))] == target[sort(names(target))]))
+
+    current <- nodeData(gr, attr = "type")
+    target <- c("high", "med", "unknown", "low", "unknown", "unknown", "unknown", "high")
+    names(target) <- c("a", "b", "c", "d", "q", "ss", "x", "y")
+    checkTrue(all(current[sort(names(current))] == target[sort(names(target))]))
+
 }
 
 test_graphBAM_addNode2 <- function(){
@@ -1321,6 +1365,45 @@ test_graphBAM_addNode2 <- function(){
     checkEquals(target, current)
 }
 
+# in version prior to 1.41.1, the propagation of existing "user" edge attributes,
+# that is, anything other than weight, failed if a new node is added which is
+# lexically less than the any nodes already in an edge.
+# the fix was simple:  see arguments to setBitCell in methods-graphBAM.R,
+# the addNode method.  test that fix here
+
+test_graphBAM_addNode_outOfAlphabeticalOrder_copyUserEdgeAttributes <- function(){
+
+    from = c("a", "b", "m", "m")
+    to   = c("b", "c", "y", "x")
+    weight=c(2.2, 2.0, 0.4, 0.2)
+    df <- data.frame(from, to, weight)
+    g <- graphBAM(df, edgemode = "directed")
+    
+    edgeDataDefaults(g, attr="color") <- "blue"
+    edgeDataDefaults(g, attr="type") <- "unknown"
+    edgeData(g, from = c("m","a"), to = c("y", "b"), attr = "color") <- c("red", "green")
+    edgeData(g, from = c("a", "b"), to = c("b", "c") , attr = "type") <- c("low", "high")
+
+    expected.edge.names <-  c("a|b", "b|c", "m|x", "m|y")
+    checkEquals(sort(names(edgeData(g, attr="color"))),
+                expected.edge.names)
+
+    checkEquals(unlist(edgeData(g, attr="color")[expected.edge.names],
+                       use.names=FALSE),
+                c("green", "blue", "blue", "red"))
+                      
+    g2 <- addNode("f", g)
+   
+       # make sure that the addition of node f does not disrupt
+       # edgeData retrieval
+    checkEquals(sort(names(edgeData(g2, attr="color"))),
+                expected.edge.names)
+    checkEquals(unlist(edgeData(g, attr="color")[expected.edge.names],
+                       use.names=FALSE),
+                c("green", "blue", "blue", "red"))
+
+}
+
 
 
 test_graphBAM_nodeUnion_Attributes <- function(use.factors=TRUE){
@@ -1451,8 +1534,9 @@ test_edgeDataUndirectedGraph <- function() {
     checkEquals(as.numeric(edgeData(g, from="a", to=c("b","c"), attr="EDA")),
                 c(1,2))
 
-    checkException(edgeData(g, from="a", to="bogus", attr="EDA"))
-    checkException(edgeData(g, from=c("a", "c"), to=c("bogus", "bagus"), attr="EDA"))
+    checkException(edgeData(g, from="a", to="bogus", attr="EDA"), silent=TRUE)
+    checkException(edgeData(g, from=c("a", "c"),
+                            to=c("bogus", "bagus"), attr="EDA"), silent=TRUE)
 }
     
 test_edgeMatrix <- function() {
@@ -1468,3 +1552,142 @@ test_edgeMatrix <- function() {
     checkEquals(rownames(mtx.dup), c("from", "to"))
     checkEquals(as.numeric(mtx.dup), c(1, 2, 2, 1))
 }
+
+  
+test_removeEdge_from_undirectedGraph <- function() {
+
+  g <- graphBAM(data.frame(from="A", to="B", weight=1))
+  g <- removeEdge(from="A", to="B", g=g)
+  checkEquals(numEdges(g), 0)
+  
+  g <- graphBAM(data.frame(from="A", to="B", weight=1))
+  g <- removeEdge(from="B", to="A", g=g)
+  checkEquals(numEdges(g), 0)
+}
+
+
+test_AM2BAM <- function(){
+
+     # test the fix for a nov 2014 bug, in which MultiGraph::.makeMDEdgeSet
+     # fails to make edge_sets in the graphBAM constructor when only 1 edge
+     # exists.   fix is at line 66 in MultiGraph.R: "drop=FALSE" added to the subset operation
+   mtx <- matrix(c(0,1,0,0), ncol=2, byrow=TRUE, dimnames=list(c("A", "B"), c("A", "B")))
+
+      # first create and check a simple (non-binary) adjacency matrix graph
+   g.am <- graphAM(mtx, edgemode="directed")
+   checkEquals(nodes(g.am),  c("A", "B"))
+   checkEquals(edgemode(g.am), "directed")
+   checkEquals(edgeNames(g.am), "A~B")
+
+     # now convert to BAM
+   g.bam <- as(g.am, "graphBAM")
+   checkEquals(nodes(g.bam),  c("A", "B"))
+   checkEquals(edgemode(g.bam), "directed")
+   checkEquals(edgeNames(g.bam), "A~B")
+
+}
+
+test_isAdjacent <- function()
+{
+  am <- adjacencyMatrix   # for shorthand
+
+  g <- graphBAM(data.frame(from="B", to="C", weight=1), edgemode="undirected")
+  checkEquals(rownames(am(g)), c("B", "C"))
+  checkEquals(colnames(am(g)), c("B", "C"))
+  checkEquals(am(g)["B","C"], 1)
+  checkEquals(am(g)["C","B"], 1)
+  checkTrue(isAdjacent(g, "B", "C"))
+  checkTrue(isAdjacent(g, "C", "B"))
+
+  checkEquals(as.numeric(edgeMatrix(g)), c(1,2))   # reciprocal edges not stored
+
+    # add a node, then an edge to the undirected graph g
+  g <- addNode("A", g)
+  checkEquals(nodes(g), c("A", "B", "C"))
+    # just one edge
+  checkEquals(sum(am(g)), 2)
+  checkEquals(am(g)["B", "C"], 1)
+  checkEquals(am(g)["C", "B"], 1)
+
+  checkTrue(isAdjacent(g, "B", "C"))
+  checkTrue(isAdjacent(g, "C", "B"))
+
+  g <- addEdge(from="C", to="A", graph=g)
+  checkEquals(sum(am(g)), 4)
+  checkEquals(am(g)["B", "C"], 1)
+  checkEquals(am(g)["C", "B"], 1)
+  checkEquals(am(g)["A", "C"], 1)
+  checkEquals(am(g)["C", "A"], 1)
+
+    # robert's bug:  both of these fail though direct inspection
+    # of either edgeMatrix or adjacencyMatrix show correct edges
+  checkTrue(isAdjacent(g, "A", "C"))
+  checkTrue(isAdjacent(g, "C", "A"))
+
+     # now verify non-reciprocity of B-C edge in a directed graph
+  gd <- graphBAM(data.frame(from="B", to="C", weight=1), edgemode="directed")
+  checkEquals(rownames(am(gd)), c("B", "C"))
+  checkEquals(colnames(am(gd)), c("B", "C"))
+  checkEquals(am(gd)["B","C"], 1)
+  checkTrue(isAdjacent(gd, "B", "C"))
+  checkTrue(!isAdjacent(gd, "C", "B"))
+
+    # add a node, then an edge to the directed graph gd
+  gd <- addNode("A", gd)
+  checkEquals(nodes(gd), c("A", "B", "C"))
+    # just one edge
+  checkEquals(sum(am(gd)), 1)
+  checkEquals(am(gd)["B", "C"], 1)
+  checkTrue(isAdjacent(gd, "B", "C"))
+
+  gd <- addEdge(from="C", to="A", graph=gd)
+  checkTrue(isAdjacent(gd, "C", "A"))
+
+} # test_isAdjacent
+
+# incomplete draft test supplied by Robert Castello (November 2014)
+test_robertCastelos_addEdge_edgeData_bug <- function() {
+
+  am <- adjacencyMatrix   # for shorthand
+  #Sys.setlocale("LC_ALL", "C")
+  #checkEquals(Sys.getlocale(), "C")
+
+  g <- graphBAM(data.frame(from="B", to="C", weight=1))
+  checkEquals(rownames(am(g)), c("B", "C"))
+  checkEquals(colnames(am(g)), c("B", "C"))
+  checkEquals(am(g)["B","C"], 1)
+
+  edgeDataDefaults(g, "x") <- NA_real_
+  g <- addNode("A", g)
+
+  checkEquals(rownames(am(g)), c("A", "B", "C"))
+  checkEquals(colnames(am(g)), c("A", "B", "C"))
+
+  g <- addEdge(from="C", to="A", graph=g)
+  checkEquals(am(g)["C", "A"], 1)
+
+  ## this one works fine
+  edgeData(g, from="A", to="C", "x") <- 10
+
+  ## however, this one breaks the code:  no longer!
+  edgeData(g, from="C", to="A", "x") <- 10
+    # ensures that no error was found in the above operations
+  checkTrue(TRUE)
+}
+
+
+
+#  Sys.setlocale("LC_ALL", "C")
+#  checkEquals(Sys.getlocale(), "C")
+#  g <- graphBAM(data.frame(from="B", to="C", weight=1))
+#  edgeDataDefaults(g, "x") <- NA_real_
+#  g <- addNode("A", g)
+#  g <- addEdge(from="C", to="A", graph=g)
+#
+#    ## this one works fine
+#  edgeData(g, from="A", to="C", "x") <- 10
+#
+#    ## however, this one breaks the code
+#  edgeData(g, from="C", to="A", "x") <- 10
+
+
